#include <SPI.h>      //RFID
#include <MFRC522.h>  //RFID
#define SS_PIN 10 //RFID
#define RST_PIN 9 //RFID
MFRC522 rfid(SS_PIN, RST_PIN);  //RFID

#define dir1  2       //Stepper Rechts
#define stepper1 3    //Stepper Rechts
#define dir2 4        //Stepper Links
#define stepper2 5    //Stepper Links

int trigPinV = A3;     //Ultraschall (US) vorne: Trig
int echoPinV = 7;     //US vorne: Echo
int trigPinL = A1;    //US Links: Trig
int echoPinL = 8;     //US Links: Echo
int trigPinR = A2;    //US Rechts: Trig
int echoPinR = 6;     //US Rechts: Echo
long cmVGL = 9, cmVGR = 9;  // globale Variablen für US links und rechts Vergleich


//Fahren:

//Geradeaus
void straight(int schritte) // Stepper bewegen sich beide nach vorne
{
  digitalWrite(dir1, HIGH); // High = Vorwärts
  digitalWrite(dir2, HIGH); // für Motor 1 und 2

  for (int n = 0; n < schritte ; n++)
  {
    digitalWrite(stepper1, HIGH); //  eine Seite der Pole wird angestellt
    digitalWrite(stepper2, HIGH);
    delay(5);                     // Verzögerung zwischen Umpolung
    digitalWrite(stepper1, LOW);  // andere Seite wird angestellt ()
    digitalWrite(stepper2, LOW);
    delay(5);
  }
}

//Rückwärts
void back (int schritte)  //beide Stepper drehen sich nach hinten
{
  digitalWrite(dir1, LOW); // rückwärts
  digitalWrite(dir2, LOW);


  for ( int j = 0; j < schritte; j++) // Bewegungsmechanismus des Schrittmotoren
  {
    digitalWrite(stepper1, LOW);
    digitalWrite(stepper2, LOW);
    delay(1);
    digitalWrite(stepper1, HIGH);
    digitalWrite(stepper2, HIGH);
    delay(1);
  }
}

//Links Drehen
void links(int schritte) //es drehen sich beide Stepper vorwärts / rückwärts
{

  digitalWrite(dir1, HIGH);   // fährt vorwärts
  digitalWrite(dir2, LOW);    // fährt rückwärts

  for (int i = 0; i < schritte; i++ ) // Bewegungs-Loop um Stepperdrehnungen durchzuführen
  {
    digitalWrite(stepper1, HIGH);  // stellt die Pole um (stepper1)
    digitalWrite(stepper2, LOW);   // stellt eine Seite aus(stepper 2)
    delay(1);
    digitalWrite(stepper1, LOW);   // stellt die Pole wieder zurück
    digitalWrite(stepper2, HIGH);
    delay(1);
  }
}

//Rechts Drehen
void rechts(int schritte)//es drehen sich beide Stepper sychron rückwärts/vorwärts
{
  digitalWrite(dir1, LOW); // Stepper rückwärts für Bewegung
  digitalWrite(dir2, HIGH); //Stepper vorwärts für Bewegung

  for (int i = 0; i < schritte; i++ )
  {
    digitalWrite(stepper1, LOW);
    digitalWrite(stepper2, HIGH);
    delay(1);
    digitalWrite(stepper1, HIGH);
    digitalWrite(stepper2, LOW);
    delay(1);
  }
}

//RFID:
bool rfidFkt() {
  straight(50);   //etwas vorwärts fahren, damit Gegenstand sicher nah genug ist
  if (rfid.PICC_IsNewCardPresent()) // neuer Tag ist verfügbar
  {
    if (rfid.PICC_ReadCardSerial()) // NUID wurde gelesen
    {
      MFRC522::PICC_Type piccType = rfid.PICC_GetType(rfid.uid.sak);
      delay(2000);
      rfid.PICC_HaltA(); // halt PICC
      rfid.PCD_StopCrypto1(); // Ende Verschlüsselung auf PCD
    }
    return 1; //Rückgabewert wahr oder falsch (richtiger Gegenstand oder nicht)
  }
  else {
    return 0;
  }
}

void setup()
{
  Serial.begin (9600);
  pinMode(trigPinV, OUTPUT);  //vorderer US Trig Pin
  pinMode(echoPinV, INPUT);   //vorderer US Eco Pin
  pinMode(trigPinL, OUTPUT);  //linker US Trig Pin
  pinMode(echoPinL, INPUT);   //linker US Echo Pin
  pinMode(trigPinR, OUTPUT);  //rechter US Trig Pin
  pinMode(echoPinR, INPUT);   //rechter US Echo Pin
  pinMode(dir1, OUTPUT);  //Fahren
  pinMode(stepper1, OUTPUT);  //Fahren
  pinMode(dir2, OUTPUT);  //Fahren
  pinMode(stepper2, OUTPUT);  //Fahren
  SPI.begin(); // init SPI bus RFID
  rfid.PCD_Init(); // init MFRC522 RFID
  pinMode(A4, OUTPUT); //Piezo


  digitalWrite(trigPinL, LOW);   //US links Höhe messen zum späteren Vergleich
  delayMicroseconds(5);
  digitalWrite(trigPinL, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigPinL, LOW);
  pinMode(echoPinL, INPUT);
  long durationVGL = pulseIn(echoPinL, HIGH);
  cmVGL = (durationVGL / 2.0) / 29.1;   // in cm berechnen und in globaler Variable speichern


  digitalWrite(trigPinR, LOW); //US rechts  Höhe messen zum späteren Vergleich
  delayMicroseconds(5);
  digitalWrite(trigPinR, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigPinR, LOW);
  pinMode(echoPinR, INPUT);
  long durationVGR = pulseIn(echoPinR, HIGH);
  cmVGR = (durationVGR / 2) / 29.1;   // in cm berechnen und in globaler Variable speichern

}

void loop()
{
  long durationV, cmV;  // Variablen für Ultraschallsensoren vorne,
  long durationL, cmL;  //links
  long durationR, cmR;  //rechts

  digitalWrite(trigPinL, LOW);   //US links Höhe messen
  delayMicroseconds(5);
  digitalWrite(trigPinL, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigPinL, LOW);
  pinMode(echoPinL, INPUT);
  durationL = pulseIn(echoPinL, HIGH);
  cmL = (durationL / 2.0) / 29.1;   // in cm berechnen

  digitalWrite(trigPinR, LOW); //US rechts  Höhe messen
  delayMicroseconds(5);
  digitalWrite(trigPinR, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigPinR, LOW);
  pinMode(echoPinR, INPUT);
  durationR = pulseIn(echoPinR, HIGH);
  cmR = (durationR / 2) / 29.1;   // in cm berechnen


  if (cmR > (cmVGR + 5) || cmL > (cmVGL + 5)) {   //wenn Differenz zu anfangs gemessenem Vergleichswert zu groß -> Abgrund
    back(200);      //wenden und drehen(um 100 Grad)
    delay(100);
    rechts(182);
  }

  else
  { straight(10);          //wenn keine Kante erkannt, geradeaus fahren

    digitalWrite(trigPinV, LOW);    //Entfernung nach vorne messen
    delayMicroseconds(5);
    digitalWrite(trigPinV, HIGH);
    delayMicroseconds(10);
    digitalWrite(trigPinV, LOW);
    pinMode(echoPinV, INPUT);
    durationV = pulseIn(echoPinV, HIGH);   //Signal lesen
    cmV = (durationV / 2) / 29.1;         //cm berechnen


    if (cmV <= 2)       //wenn Gegenstand 2 cm entfernt von US-Sensor ist, RFID-Leser anschalten
    {
      if ( rfidFkt()) //wenn richtiger Gegenstand
      {
        tone(A4, 440);   //Ton ausgeben
        delay(3000);
        noTone(A4);
        while (1);  //"Suche beenden", Leerlauf
      }
      else {    //falscher Gegenstand:
        back(200);      //wenden und drehen(um 100 Grad)
        rechts(182);
      }
    }
  }
}
